@@ -1,22 +1,16 @@
 """Scanning tools for finding mugs."""
+
 from __future__ import annotations
 
 import asyncio
 import contextlib
 import logging
-import sys
 from typing import TYPE_CHECKING, Any
 
 from bleak import BleakScanner
 
 from .consts import DEVICE_SERVICE_UUIDS, IS_LINUX
 
-if sys.version_info < (3, 11):
-    # library required before Python 3.11
-    import async_timeout
-else:
-    async_timeout = asyncio
-
 if TYPE_CHECKING:
     from bleak.backends.device import BLEDevice
     from bleak.backends.scanner import AdvertisementData
@@ -36,12 +30,23 @@ def build_scanner_kwargs(adapter: str | None = None) -> dict[str, Any]:
     return kwargs | {"adapter": adapter} if adapter else kwargs
 
 
-async def discover_mugs(
+async def discover_devices(
     mac: str | None = None,
     adapter: str | None = None,
     wait: int = 5,
 ) -> list[tuple[BLEDevice, AdvertisementData]]:
-    """Discover new mugs in pairing mode."""
+    """
+    Discover new devices in pairing mode.
+
+    Example:
+    -------
+        ```python
+        devices = await discover_devices()
+        for device, advertisement in devices:
+            print(device.address, advertisement)
+        ```
+
+    """
     async with BleakScanner(**build_scanner_kwargs(adapter)) as scanner:
         await asyncio.sleep(wait)
         return [
@@ -51,17 +56,26 @@ async def discover_mugs(
         ]
 
 
-async def find_mug(
+async def find_device(
     mac: str | None = None,
     adapter: str | None = None,
-    timeout: int = DEFAULT_TIMEOUT,
+    timeout: int = DEFAULT_TIMEOUT,  # noqa: ASYNC109
 ) -> tuple[BLEDevice, AdvertisementData] | tuple[None, None]:
-    """Find a mug."""
+    """
+    Find a device that has previously been discovered.
+
+    Example:
+    -------
+        ```python
+        device = await find_device("my:mac:addr")
+        ```
+
+    """
     if mac is not None:
         mac = mac.lower()
     async with BleakScanner(**build_scanner_kwargs(adapter)) as scanner:
         with contextlib.suppress(asyncio.TimeoutError):
-            async with async_timeout.timeout(timeout):
+            async with asyncio.timeout(timeout):
                 async for device, advertisement in scanner.advertisement_data():
                     if (not mac and device.name and device.name.startswith("Ember")) or (
                         mac and device.address.lower() == mac
