@@ -1,4 +1,5 @@
 """Tests for `ember_mug.mug connections`."""
+
 from __future__ import annotations
 
 from datetime import datetime
@@ -10,33 +11,36 @@ from bleak import BleakError
 from bleak.backends.device import BLEDevice
 
 from ember_mug.consts import (
-    EMBER_MUG,
-    EXTRA_ATTRS,
     INITIAL_ATTRS,
     UPDATE_ATTRS,
+    DeviceModel,
     MugCharacteristic,
     TemperatureUnit,
     VolumeLevel,
 )
-from ember_mug.data import Colour, Model
+from ember_mug.data import Colour, ModelInfo
 from ember_mug.mug import EmberMug
+from tests.conftest import TEST_MUG_ADVERTISEMENT
 
 if TYPE_CHECKING:
 
     class MockMug(EmberMug):
+        """For type checking."""
+
         _client: AsyncMock
 
 
 @patch("ember_mug.mug.IS_LINUX", True)
 async def test_adapter_with_bluez(ble_device: BLEDevice):
-    mug = EmberMug(ble_device, adapter="hci0")
+    mug = EmberMug(ble_device, ModelInfo(), adapter="hci0")
     assert mug._client_kwargs["adapter"] == "hci0"
 
 
 @patch("ember_mug.mug.IS_LINUX", False)
 async def test_adapter_without_bluez(ble_device: BLEDevice):
-    with pytest.raises(ValueError):
-        EmberMug(ble_device, adapter="hci0")
+    expected_message = "The adapter option is only valid for the Linux BlueZ Backend."
+    with pytest.raises(ValueError, match=expected_message):
+        EmberMug(ble_device, ModelInfo(), adapter="hci0")
 
 
 @patch("ember_mug.mug.EmberMug.subscribe")
@@ -74,11 +78,10 @@ async def test_connect_error(
     ember_mug: MockMug,
 ) -> None:
     ember_mug._client = None  # type: ignore[assignment]
-    mock_establish_connection.side_effect = BleakError
-    with pytest.raises(BleakError):
+    mock_establish_connection.side_effect = BleakError("bleak-error")
+    with pytest.raises(BleakError, match="bleak-error"):
         await ember_mug._ensure_connection()
-    msg, device, exception = mock_logger.debug.mock_calls[1].args
-    assert msg == "%s: Failed to connect to the mug: %s"
+    device, exception = mock_logger.debug.mock_calls[1].args[1:]
     assert device == ember_mug.device
     assert isinstance(exception, BleakError)
 
@@ -159,7 +162,7 @@ async def test_read(
     ember_mug: MockMug,
 ) -> None:
     with patch.object(ember_mug, "_ensure_connection", AsyncMock()):
-        ember_mug._client.read_gatt_char.return_value = b"TEST"
+        ember_mug._client.read_gatt_char = AsyncMock(return_value=b"TEST")
         await ember_mug._read(MugCharacteristic.MUG_NAME)
         ember_mug._client.read_gatt_char.assert_called_with(
             MugCharacteristic.MUG_NAME.uuid,
@@ -175,6 +178,7 @@ async def test_read(
 async def test_write(mock_logger: Mock, ember_mug: MockMug) -> None:
     with patch.object(ember_mug, "_ensure_connection", AsyncMock()):
         test_name = bytearray(b"TEST")
+        ember_mug._client.write_gatt_char = AsyncMock()
         await ember_mug._write(
             MugCharacteristic.MUG_NAME,
             test_name,
@@ -190,8 +194,8 @@ async def test_write(mock_logger: Mock, ember_mug: MockMug) -> None:
         )
 
         ember_mug._client = AsyncMock()
-        ember_mug._client.write_gatt_char.side_effect = BleakError
-        with pytest.raises(BleakError):
+        ember_mug._client.write_gatt_char = AsyncMock(side_effect=BleakError("bleak-error"))
+        with pytest.raises(BleakError, match="bleak-error"):
             await ember_mug._write(
                 MugCharacteristic.MUG_NAME,
                 test_name,
@@ -207,21 +211,38 @@ async def test_write(mock_logger: Mock, ember_mug: MockMug) -> None:
         assert isinstance(exception, BleakError)
 
 
-def test_set_device(ember_mug: MockMug) -> None:
+def test_ble_event_callback(ember_mug: MockMug) -> None:
     new_device = BLEDevice(
         address="BA:36:a5:be:88:cb",
         name="Ember Ceramic Mug",
         details={},
         rssi=1,
     )
+    ember_mug.data.model_info.model = None
     assert ember_mug.device.address != new_device.address
-    ember_mug.set_device(new_device)
+    ember_mug.ble_event_callback(new_device, TEST_MUG_ADVERTISEMENT)
+    assert ember_mug.model_name == DeviceModel.MUG_2_10_OZ
     assert ember_mug.device.address == new_device.address
 
 
+def test_can_write(ember_mug: MockMug) -> None:
+    ember_mug.data.udsk = "non-empty"
+    assert ember_mug.can_write is True
+
+    ember_mug.data.udsk = None
+    assert ember_mug.can_write is False
+
+
+def test_has_attribute(ember_mug: MockMug) -> None:
+    ember_mug.data.model_info.model = DeviceModel.CUP_6_OZ
+    assert ember_mug.has_attribute("name") is False
+    ember_mug.data.model_info = ModelInfo(DeviceModel.MUG_2_10_OZ)
+    assert ember_mug.has_attribute("name") is True
+
+
 async def test_get_mug_meta(ember_mug: MockMug) -> None:
     with patch.object(ember_mug, "_ensure_connection", AsyncMock()):
-        ember_mug._client.read_gatt_char.return_value = b"Yw====-ABCDEFGHIJ"
+        ember_mug._client.read_gatt_char = AsyncMock(return_value=b"Yw====-ABCDEFGHIJ")
         meta = await ember_mug.get_meta()
         assert meta.mug_id == "WXc9PT09"
         assert meta.serial_number == "ABCDEFGHIJ"
@@ -230,7 +251,7 @@ async def test_get_mug_meta(ember_mug: MockMug) -> None:
 
 async def test_get_mug_battery(ember_mug: MockMug) -> None:
     with patch.object(ember_mug, "_ensure_connection", AsyncMock()):
-        ember_mug._client.read_gatt_char.return_value = b"5\x01"
+        ember_mug._client.read_gatt_char = AsyncMock(return_value=b"5\x01")
         battery = await ember_mug.get_battery()
         assert battery.percent == 53.00
         assert battery.on_charging_base is True
@@ -239,7 +260,7 @@ async def test_get_mug_battery(ember_mug: MockMug) -> None:
 
 async def test_get_mug_led_colour(ember_mug: MockMug) -> None:
     with patch.object(ember_mug, "_ensure_connection", AsyncMock()):
-        ember_mug._client.read_gatt_char.return_value = b"\xf4\x00\xa1\xff"
+        ember_mug._client.read_gatt_char = AsyncMock(return_value=b"\xf4\x00\xa1\xff")
         colour = await ember_mug.get_led_colour()
         assert colour.as_hex() == "#f400a1"
         ember_mug._client.read_gatt_char.assert_called_once_with(MugCharacteristic.LED.uuid)
@@ -247,6 +268,7 @@ async def test_get_mug_led_colour(ember_mug: MockMug) -> None:
 
 async def test_set_mug_led_colour(ember_mug: MockMug) -> None:
     mock_ensure_connection = AsyncMock()
+    ember_mug._client.write_gatt_char = AsyncMock()
     with patch.object(ember_mug, "_ensure_connection", mock_ensure_connection):
         await ember_mug.set_led_colour(Colour(244, 0, 161))
         mock_ensure_connection.assert_called_once()
@@ -257,8 +279,9 @@ async def test_set_mug_led_colour(ember_mug: MockMug) -> None:
 
 
 async def test_set_volume_level_travel_mug(ember_mug: MockMug) -> None:
-    ember_mug.is_travel_mug = True
+    ember_mug.data.model_info.model = DeviceModel.TRAVEL_MUG_12_OZ
     mock_ensure_connection = AsyncMock()
+    ember_mug._client.write_gatt_char = AsyncMock()
     with patch.object(ember_mug, "_ensure_connection", mock_ensure_connection):
         await ember_mug.set_volume_level(VolumeLevel.HIGH)
         mock_ensure_connection.assert_called_once()
@@ -280,7 +303,8 @@ async def test_set_volume_level_travel_mug(ember_mug: MockMug) -> None:
 async def test_set_volume_level_mug(ember_mug: MockMug) -> None:
     mock_ensure_connection = AsyncMock()
     with patch.object(ember_mug, "_ensure_connection", mock_ensure_connection):
-        with pytest.raises(NotImplementedError):
+        error = "The mug does not have the volume_level attribute"
+        with pytest.raises(NotImplementedError, match=error):
             await ember_mug.set_volume_level(VolumeLevel.HIGH)
         mock_ensure_connection.assert_not_called()
         ember_mug._client.write_gatt_char.assert_not_called()
@@ -288,13 +312,21 @@ async def test_set_volume_level_mug(ember_mug: MockMug) -> None:
 
 async def test_get_mug_target_temp(ember_mug: MockMug) -> None:
     with patch.object(ember_mug, "_ensure_connection", AsyncMock()):
-        ember_mug._client.read_gatt_char.return_value = b"\xcd\x15"
+        ember_mug._client.read_gatt_char = AsyncMock(return_value=b"\xcd\x15")
         assert (await ember_mug.get_target_temp()) == 55.81
         ember_mug._client.read_gatt_char.assert_called_once_with(MugCharacteristic.TARGET_TEMPERATURE.uuid)
 
 
-async def test_set_mug_target_temp(ember_mug: MockMug) -> None:
+async def test_set_mug_target_temp_celsius(ember_mug: MockMug) -> None:
     mock_ensure_connection = AsyncMock()
+    ember_mug._client.write_gatt_char = AsyncMock()
+    error = "Temperature should be between 49 and 63 or 0."
+    with pytest.raises(ValueError, match=error):
+        await ember_mug.set_target_temp(10)
+
+    with pytest.raises(ValueError, match=error):
+        await ember_mug.set_target_temp(100)
+
     with patch.object(ember_mug, "_ensure_connection", mock_ensure_connection):
         await ember_mug.set_target_temp(55.81)
         mock_ensure_connection.assert_called_once()
@@ -304,39 +336,66 @@ async def test_set_mug_target_temp(ember_mug: MockMug) -> None:
         )
 
 
+async def test_set_mug_target_temp_fahrenheit(ember_mug: MockMug) -> None:
+    mock_ensure_connection = AsyncMock()
+    ember_mug._client.write_gatt_char = AsyncMock()
+    ember_mug.data.use_metric = False
+    error = "Temperature should be between 120 and 145 or 0."
+
+    with pytest.raises(ValueError, match=error):
+        await ember_mug.set_target_temp(50)
+
+    with pytest.raises(ValueError, match=error):
+        await ember_mug.set_target_temp(200)
+
+    with patch.object(ember_mug, "_ensure_connection", mock_ensure_connection):
+        await ember_mug.set_target_temp(132.45)
+        mock_ensure_connection.assert_called_once()
+        ember_mug._client.write_gatt_char.assert_called_once_with(
+            MugCharacteristic.TARGET_TEMPERATURE.uuid,
+            bytearray(b"\xcc\x15"),
+        )
+
+
 async def test_get_mug_current_temp(ember_mug: MockMug) -> None:
     with patch.object(ember_mug, "_ensure_connection", AsyncMock()):
-        ember_mug._client.read_gatt_char.return_value = b"\xcd\x15"
+        ember_mug._client.read_gatt_char = AsyncMock(return_value=b"\xcd\x15")
         assert (await ember_mug.get_current_temp()) == 55.81
         ember_mug._client.read_gatt_char.assert_called_once_with(MugCharacteristic.CURRENT_TEMPERATURE.uuid)
 
 
 async def test_get_mug_liquid_level(ember_mug: MockMug) -> None:
     with patch.object(ember_mug, "_ensure_connection", AsyncMock()):
-        ember_mug._client.read_gatt_char.return_value = b"\n"
+        ember_mug._client.read_gatt_char = AsyncMock(return_value=b"\n")
         assert (await ember_mug.get_liquid_level()) == 10
         ember_mug._client.read_gatt_char.assert_called_once_with(MugCharacteristic.LIQUID_LEVEL.uuid)
 
 
 async def test_get_mug_liquid_state(ember_mug: MockMug) -> None:
     with patch.object(ember_mug, "_ensure_connection", AsyncMock()):
-        ember_mug._client.read_gatt_char.return_value = b"\x06"
+        ember_mug._client.read_gatt_char = AsyncMock(return_value=b"\x06")
         assert (await ember_mug.get_liquid_state()) == 6
         ember_mug._client.read_gatt_char.assert_called_once_with(MugCharacteristic.LIQUID_STATE.uuid)
 
 
 async def test_get_mug_name(ember_mug: MockMug) -> None:
     with patch.object(ember_mug, "_ensure_connection", AsyncMock()):
-        ember_mug._client.read_gatt_char.return_value = b"Mug Name"
+        ember_mug._client.read_gatt_char = AsyncMock(return_value=b"Mug Name")
         assert (await ember_mug.get_name()) == "Mug Name"
         ember_mug._client.read_gatt_char.assert_called_once_with(MugCharacteristic.MUG_NAME.uuid)
 
+        ember_mug.data.model_info = ModelInfo(DeviceModel.CUP_6_OZ)
+        with pytest.raises(NotImplementedError):
+            await ember_mug.get_name()
+
 
 async def test_set_mug_name(ember_mug: MockMug) -> None:
-    with patch.object(ember_mug, "_ensure_connection", AsyncMock()), pytest.raises(ValueError):
+    invalid_name = "Name cannot contain any special characters and must be 16 characters or less"
+    with patch.object(ember_mug, "_ensure_connection", AsyncMock()), pytest.raises(ValueError, match=invalid_name):
         await ember_mug.set_name("Hé!")
 
     mock_ensure_connection = AsyncMock()
+    ember_mug._client.write_gatt_char = AsyncMock()
     with patch.object(ember_mug, "_ensure_connection", mock_ensure_connection):
         await ember_mug.set_name("Mug name")
         mock_ensure_connection.assert_called()
@@ -345,16 +404,22 @@ async def test_set_mug_name(ember_mug: MockMug) -> None:
             bytearray(b"Mug name"),
         )
 
+        ember_mug.data.model_info = ModelInfo(DeviceModel.CUP_6_OZ)
+        error = "The cup does not have the name attribute"
+        with pytest.raises(NotImplementedError, match=error):
+            await ember_mug.set_name("Test")
+
 
 async def test_get_mug_udsk(ember_mug: MockMug) -> None:
     with patch.object(ember_mug, "_ensure_connection", AsyncMock()):
-        ember_mug._client.read_gatt_char.return_value = b"abcd12345"
+        ember_mug._client.read_gatt_char = AsyncMock(return_value=b"abcd12345")
         assert (await ember_mug.get_udsk()) == "YWJjZDEyMzQ1"
         ember_mug._client.read_gatt_char.assert_called_once_with(MugCharacteristic.UDSK.uuid)
 
 
 async def test_set_mug_udsk(ember_mug: MockMug) -> None:
     mock_ensure_connection = AsyncMock()
+    ember_mug._client.write_gatt_char = AsyncMock()
     with patch.object(ember_mug, "_ensure_connection", mock_ensure_connection):
         await ember_mug.set_udsk("abcd12345")
         mock_ensure_connection.assert_called_once()
@@ -366,26 +431,26 @@ async def test_set_mug_udsk(ember_mug: MockMug) -> None:
 
 async def test_get_mug_dsk(ember_mug: MockMug) -> None:
     with patch.object(ember_mug, "_ensure_connection", AsyncMock()):
-        ember_mug._client.read_gatt_char.return_value = b"abcd12345"
+        ember_mug._client.read_gatt_char = AsyncMock(return_value=b"abcd12345")
         assert (await ember_mug.get_dsk()) == "YWJjZDEyMzQ1"
-        ember_mug._client.read_gatt_char.return_value = b"something else"
+        ember_mug._client.read_gatt_char = AsyncMock(return_value=b"something else")
         assert (await ember_mug.get_dsk()) == "c29tZXRoaW5nIGVsc2U="
         ember_mug._client.read_gatt_char.assert_called_with(MugCharacteristic.DSK.uuid)
 
 
 async def test_get_mug_temperature_unit(ember_mug: MockMug) -> None:
     with patch.object(ember_mug, "_ensure_connection", AsyncMock()):
-        ember_mug._client.read_gatt_char.return_value = b"\x01"
+        ember_mug._client.read_gatt_char = AsyncMock(return_value=b"\x01")
         assert (await ember_mug.get_temperature_unit()) == TemperatureUnit.FAHRENHEIT
         ember_mug._client.read_gatt_char.assert_called_once_with(MugCharacteristic.TEMPERATURE_UNIT.uuid)
-        ember_mug._client.read_gatt_char.reset_mock()
-        ember_mug._client.read_gatt_char.return_value = b"\x00"
+        ember_mug._client.read_gatt_char = AsyncMock(return_value=b"\x00")
         assert (await ember_mug.get_temperature_unit()) == TemperatureUnit.CELSIUS
         ember_mug._client.read_gatt_char.assert_called_once_with(MugCharacteristic.TEMPERATURE_UNIT.uuid)
 
 
 async def test_set_mug_temperature_unit(ember_mug: MockMug) -> None:
     mock_ensure_connection = AsyncMock()
+    ember_mug._client.write_gatt_char = AsyncMock()
     with patch.object(ember_mug, "_ensure_connection", mock_ensure_connection):
         await ember_mug.set_temperature_unit(TemperatureUnit.CELSIUS)
         mock_ensure_connection.assert_called_once()
@@ -400,7 +465,7 @@ async def test_mug_ensure_correct_unit(ember_mug: MockMug) -> None:
         ember_mug.data.temperature_unit = TemperatureUnit.CELSIUS
         ember_mug.data.use_metric = True
         mock_set_temp = AsyncMock(return_value=None)
-        with patch.object(ember_mug, 'set_temperature_unit', mock_set_temp):
+        with patch.object(ember_mug, "set_temperature_unit", mock_set_temp):
             await ember_mug.ensure_correct_unit()
             mock_set_temp.assert_not_called()
             ember_mug.data.temperature_unit = TemperatureUnit.FAHRENHEIT
@@ -410,14 +475,14 @@ async def test_mug_ensure_correct_unit(ember_mug: MockMug) -> None:
 
 async def test_get_mug_battery_voltage(ember_mug: MockMug) -> None:
     with patch.object(ember_mug, "_ensure_connection", AsyncMock()):
-        ember_mug._client.read_gatt_char.return_value = b"\x01"
+        ember_mug._client.read_gatt_char = AsyncMock(return_value=b"\x01")
         assert (await ember_mug.get_battery_voltage()) == 1
         ember_mug._client.read_gatt_char.assert_called_once_with(MugCharacteristic.CONTROL_REGISTER_DATA.uuid)
 
 
 async def test_get_mug_date_time_zone(ember_mug: MockMug) -> None:
     with patch.object(ember_mug, "_ensure_connection", AsyncMock()):
-        ember_mug._client.read_gatt_char.return_value = b"c\x0f\xf6\x00"
+        ember_mug._client.read_gatt_char = AsyncMock(return_value=b"c\x0f\xf6\x00")
         date_time = await ember_mug.get_date_time_zone()
         assert isinstance(date_time, datetime)
         assert date_time.timestamp() == 1661990400.0
@@ -426,7 +491,7 @@ async def test_get_mug_date_time_zone(ember_mug: MockMug) -> None:
 
 async def test_read_firmware(ember_mug: MockMug) -> None:
     with patch.object(ember_mug, "_ensure_connection", AsyncMock()):
-        ember_mug._client.read_gatt_char.return_value = b"c\x01\x80\x00\x12\x00"
+        ember_mug._client.read_gatt_char = AsyncMock(return_value=b"c\x01\x80\x00\x12\x00")
         firmware = await ember_mug.get_firmware()
         assert firmware.version == 355
         assert firmware.hardware == 128
@@ -435,17 +500,9 @@ async def test_read_firmware(ember_mug: MockMug) -> None:
 
 
 async def test_mug_update_initial(ember_mug: MockMug) -> None:
-    no_extra = INITIAL_ATTRS - EXTRA_ATTRS
-
     mock_update = AsyncMock(return_value={})
     with patch.multiple(ember_mug, _ensure_connection=AsyncMock(), _update_multiple=mock_update):
-        ember_mug.data.model = Model(EMBER_MUG, include_extra=False)
-        assert (await ember_mug.update_initial()) == {}
-        mock_update.assert_called_once_with(no_extra)
-
-        # Try with extra
-        mock_update.reset_mock()
-        ember_mug.data.model = Model(EMBER_MUG, include_extra=True)
+        ember_mug.data.model_info = ModelInfo()
         assert (await ember_mug.update_initial()) == {}
         mock_update.assert_called_once_with(INITIAL_ATTRS)
 
@@ -453,34 +510,29 @@ async def test_mug_update_initial(ember_mug: MockMug) -> None:
 async def test_mug_update_all(ember_mug: MockMug) -> None:
     mock_update = AsyncMock(return_value={})
     with patch.multiple(ember_mug, _ensure_connection=AsyncMock(), _update_multiple=mock_update):
-        assert (await ember_mug.update_all()) == {}
-        mock_update.assert_called_once_with(UPDATE_ATTRS - EXTRA_ATTRS)
-
-        # Try with extras
-        mock_update.reset_mock()
         assert (await ember_mug.update_all()) == {}
         mock_update.assert_called_once_with(UPDATE_ATTRS)
 
 
 async def test_mug_update_multiple(ember_mug: MockMug) -> None:
     mock_get_name = AsyncMock(return_value="name")
-    mock_update_info = AsyncMock()
 
-    with patch.multiple(ember_mug, _ensure_connection=AsyncMock(), get_name=mock_get_name):
-        with patch.object(ember_mug.data, 'update_info', mock_update_info):
-            await ember_mug._update_multiple({"name"})
-            mock_get_name.assert_called_once()
-            mock_update_info.assert_called_once_with(name="name")
+    with (
+        patch.multiple(ember_mug, get_name=mock_get_name),
+        patch.object(ember_mug.data, "update_info") as mock_update_info,
+    ):
+        await ember_mug._update_multiple({"name"})
+        mock_get_name.assert_called_once()
+        mock_update_info.assert_called_once_with(name="name")
 
 
 async def test_mug_update_queued_attributes(ember_mug: MockMug) -> None:
     mock_get_name = AsyncMock(return_value="name")
-    mock_update_info = AsyncMock()
 
-    with patch.multiple(ember_mug, _ensure_connection=AsyncMock(), get_name=mock_get_name):
+    with patch.multiple(ember_mug, get_name=mock_get_name):
         ember_mug._queued_updates = set()
         assert (await ember_mug.update_queued_attributes()) == []
-        with patch.object(ember_mug.data, 'update_info', mock_update_info):
+        with patch.object(ember_mug.data, "update_info") as mock_update_info:
             ember_mug._queued_updates = {"name"}
             await ember_mug.update_queued_attributes()
             mock_update_info.assert_called_once_with(name="name")
