@@ -1,9 +1,10 @@
 """Tests for `ember_mug.consts`."""
+
 from __future__ import annotations
 
 from uuid import UUID
 
-from ember_mug.consts import LiquidState, MugCharacteristic
+from ember_mug.consts import LiquidState, MugCharacteristic, VolumeLevel
 
 
 def test_mug_uuids() -> None:
@@ -12,7 +13,7 @@ def test_mug_uuids() -> None:
 
 
 def test_liquid_state() -> None:
-    assert LiquidState(0).label == "Unknown"
+    assert LiquidState(0).label == "Standby"
     assert LiquidState(1).label == "Empty"
     assert LiquidState(2).label == "Filling"
     assert LiquidState(3).label == "Cold (No control)"
@@ -20,3 +21,13 @@ def test_liquid_state() -> None:
     assert LiquidState(5).label == "Heating"
     assert LiquidState(6).label == "Perfect"
     assert LiquidState(7).label == "Warm (No control)"
+
+
+def test_volume_level() -> None:
+    assert VolumeLevel.from_state(0) == VolumeLevel.LOW
+    assert VolumeLevel.from_state(1) == VolumeLevel.MEDIUM
+    assert VolumeLevel.from_state(2) == VolumeLevel.HIGH
+
+    assert VolumeLevel.HIGH.state == 2
+    assert VolumeLevel.MEDIUM.state == 1
+    assert VolumeLevel.LOW.state == 0
